
/********************************************* PROJEKT **************************************************************

Predmet:      Sustavi za praćenje i vođenje procesa
Projekt:      Mjerenje plinova u kućanstvu
Autor:        Zvonimir Hölbling
JMBAG:        0036446736
Sveučilište:  Fakultet elektrotehnike i računarstva, 10000 Zagreb

  
                                                                                                         29-05-2014
********************************************************************************************************************/
 
/*************************** TGS-4161 ******************************************************************************/
#define           SEN_PIN                    (0)     //analogni ulaz
#define           DC_GAIN                    (9.2)   //gain pojačala
 
/*************************** TGS-4161 ******************************************************************************/
#define              N                       (50)    //broj uzoraka po intervalu
#define              A                       (10)    //broj uzoraka za procjenu medijana dV/dt
#define         SAMPLE_TIME                  (5)    //vrijeme između svakog intervala
                                                     //(u milisekundama) 
 
/*************************** TGS-4161 *******************************************************************************/

#define              V0                      (340)   //izlazna vrijednost senzora [mV] uz koncentraciju CO2 = 400 ppm
#define              G1                      (400)   //donja granica CO2 koncentracija
#define              G2                      (1000)  //2. granica CO2 koncentracija
#define              G3                      (3000)  //3. granica CO2 koncentracija
#define              G4                      (10000) //gornja granica CO2 koncentracija (10,000 ppm = 1% zraka)
#define           THRESHOLD                  (5000)  //prag u [ppm] za nisku razinu CO2 

/************************* MQ-2 *************************************************************************************/
#define         MQ_PIN                       (2)     //analogni ulaz za detekciju dima
#define         RL_VALUE                     (5)     //RL u kOhm
#define         RO_CLEAN_AIR_FACTOR          (9.83)  //(otpor senzora u čistom zraku)/RO,dobiven iz grafikona u datasheetu

/************************* MQ-2 *************************************************************************************/
#define         CALIBARAION_SAMPLE_TIMES     (50)    //broj sampleova za kalibraciju
#define         CALIBRATION_SAMPLE_INTERVAL  (500)   //vremenski interval(ms) između svakog samplea u kalibracijskoj fazi
#define         READ_SAMPLE_INTERVAL         (50)    //broj sampleova u normalnoj operaciji
#define         READ_SAMPLE_TIMES            (5)     //vremenski razmak (ms) između svakog samplea u normalnoj fazi
/********************************************************************************************************************/

#include <SoftwareSerial.h>         // library za serijsku komunikaciju s GSM shiledom
SoftwareSerial  Sim900Serial(2, 3); // definiranje Rx i Tx pinova na GSM shiledu 
int POWER = 7;                      // broj pina preko kojeg palimo GSM modul

int green = 12;
float voltage = 0;
float LastVoltage = 0;
float dV = 0;
unsigned long lastTime = 0;
unsigned long bigTime = 0;
unsigned long dt = 1000; // dt u milisekundama
int a = 0;
int b = 0;
int c = 0;
// MQ-2

float           SmokeCurve[3] = {2.3,0.53,-0.44};   // linearna aproksimacija krivulje ovisnosti
                                                    // omjera otpra senzora o koncentraciji CO2
float                      Ro = 10;                 // Ro inicijalizirano na 10 kOhma


float smoke=0;
float LastSmoke=0;
float dS=0;
unsigned long time = 0;


void setup(){

  // definiranje parametara GSM veze
  Sim900Serial.begin(115200);              // the GPRS baud rate  
  delay(2000);
  Sim900Serial.println("AT+IPR=19200");    // Set the baud rate
  delay(500);
  Sim900Serial.begin(19200);               // the GPRS baud rate   
  delay(1000);
  // ukljucivanje GSM modula
  digitalWrite(POWER, HIGH);
  delay(3000);
  digitalWrite(POWER, LOW);
  delay(2000);   

  pinMode(green, OUTPUT);                  // inicijalizacija zelene diode - simulira otvaranje prozora
  
  Serial.begin(9600);                              //baudrate = 9600bps
  Serial.print("**************** MJERENJE CO2 I DIMA U ZRAKU ************************\n");  
  Serial.print("Calibrating...\n");                
  Ro = MQCalibration(MQ_PIN);                       // Kalibracija senzora MQ-2 - mora biti u čistom zraku u to vrijeme

  Serial.print("Calibration (MQ-2) is done...\n"); 
  Serial.print("Ro=");
  Serial.print(Ro);
  Serial.print("kohm");
  Serial.print("\n");
  Serial.println();
  Serial.print("*********************************************************************\n");                
}


void loop(){
  
    int co2;
    float volts;
    float delta;
   
    volts = TGSRead(SEN_PIN);
    Serial.print("\nU: ");
    Serial.print(volts); 
    Serial.print("mV   ");
    
    delta = V0-(volts/DC_GAIN);

    co2 = TGSPercentage(volts,delta);
    smoke = MQGetPercentage(MQRead(MQ_PIN)/Ro,SmokeCurve);
    
    Serial.print("CO2: ");
    if (co2 == -1) {
        Serial.print("<400");
    } else {
        Serial.print(co2);
        Serial.print("ppm");
    }
    // MQ-2
    Serial.print("    ");   
    Serial.print("SMOKE:"); 
    Serial.print(smoke);
    Serial.print( "ppm" );
    
 /****************************** Brzina promjene napona ***************************************
 ***************************************** dV/dt **********************************************/  

  if(millis() - bigTime >= 10*dt){
    bigTime = millis();
    float voltage = volts;
    dV = voltage - LastVoltage;
    Serial.print("    dV: ");
    Serial.print(dV);
    Serial.print("mV");
    LastVoltage = voltage;
    a++;
 
 if(a>=6){
    if(dV< -50){b++;}
    else if (b==0){b=0;}
    else{b--;}
    
 if(b>=3){
    b=0;
    Serial.print("\n                             Vise ljudi je prisutno u prostoriji");}
 }}

/************************************* MQ-2 ***********************************
*******************************************************************************/
 
   if(millis() - time >= dt){
    time= millis();
    dS = smoke - LastSmoke;
    if(dS){
    Serial.print("    dS: ");
    Serial.print(dS);
    }
    LastSmoke = smoke;
    }
    
    if(smoke>500){
          Serial.print("\n                             Salji obavijest da je pozar u prostoriji!!");
    
      if(green==HIGH){
        digitalWrite(green,LOW);                      // ako je potencijalni požar, zatvori prozore (ako su otvoreni) 
                                                 // kako bi maksimalno usporili širenje vatre
    }  
   
          delay(3000);
          Sim900Serial.println("ATD00385917903929;");   // definiramo broj koji zelimo nazvati "ATD+broj;"
                                                        // +broj definiramo u standardnom obliku
                                                        // za Hrvatsku 00385*********
          delay(15000);                                 // ukoliko dode do detekcije drzi izlaz  HIGH 20sec
          Sim900Serial.println("ATH");                  // zavrsi poziv

}
/************* LED ****************************************************************/
  if (co2 > THRESHOLD) {                             
      c++;
      if(c>5){
      digitalWrite(green, HIGH);                        // kada CO2 razina bude veća od praga, otvaraju se prozori
      c=0;}                                             // no tek nakon 5. provjere radi istitravanja
  }
  else{
       digitalWrite(green,LOW);
     }
  delay(500);
}

/*********************************************************************************
                              FUNKCIJE
*********************************************************************************/

/**************************** TGSRead ********************************************
Ulaz:   sen_pin - analogni kanal (0).
Izlaz:  Izlaz pojačala (TLC271CP).
Opis:   Očitavanje napona s izlaza pojačala.
*********************************************************************************/ 
float TGSRead(int sen_pin){
    int i;
    int j;
    float valArray[N];
    float voltsMean=0;
    
    for (i=0;i<N;i++) {
      valArray[i] = analogRead(sen_pin);
      delay(SAMPLE_TIME);
    }
 
    voltsMean = ModeFilter(valArray, N);   
    return voltsMean*5000/1024;
 }
 
/***************************** Mode Filter ****************************************
Ulaz:   Vektor vrijednosti val[] duljine n.
Izlaz:  Usrednjena vrijednost od medijana vrijednosti ulaznog vektora.
Opis:   Naziv filtra je Mode Filter jer radi optimizaciju medijana i usrednjavanja
        te daje bolju točnost. Funkcija prvo radi medijan svih vrijednosti zadanog
        vektora. Nakon toga dodatno računa srednju vrijednost iz sredine medijan
        vektora, tj. između njegovih srednjih n/10 elemenata.
**********************************************************************************/
float ModeFilter(float *val, int n){
    int i;
    int j;
    float sort[n];
    
  for (i=0; i<n; i++) {
     if(val[i] < sort[0] || i==0){
        j=0; // inicijalizacija prve pozicije
     }
     else{
       for(j=1; j<i; j++){
          if(sort[j-1] <= val[i] && sort[j] >= val[i]){
            // traženje pozicije gdje je jedan element manji, 
            // a sljedeći element veći od trenutne vrijednosti
            break;
          }
       }
     }
     for(int k=i; k>j; k--){
       // pomicanje svih vrijednosti za jedan iznad trenutne očitane vrijednosti
       sort[k] = sort[k-1];
     }
     sort[j] = val[i]; // unos trenutnog očitavanja
   }
   // Usrednjavanje sredine median vrijednosti
   float returnval = 0;
   for(int i=n/2-n/10; i<(n/2+n/10); i++){
     returnval +=sort[i];
   }
   return float(returnval/(n/5)); 
}
  
/**************************** TGSPercentage ************************************
Ulaz:    volts - TGS-4161 izlaz izmjeren u voltima.
Izlaz:   Koncentracija CO2 izražena u ppm.
Opis:    Procjena promjene CO2 koncentracije u zraku ovisno o promjeni napona na
         mjerenom senzoru.
********************************************************************************/ 
float  TGSPercentage(float volts, float delta){

     if ((volts/DC_GAIN )>=V0) {
      return -1;
   } else if (delta <= 56){
       return (G1 + (G2 - G1)*(0.0178757 * delta));
   }else if (delta <= 112){
       return (G2 + (G3 - G2)*(0.0178757 * (delta - 56)));
   }else{
       return (G3 + (G4 - G3)*(0.0178757 * (delta - 112)));
   }
 }
/****************** MQResistanceCalculation ****************************************
Ulaz:   raw_adc - nominalna vrijednost očitana s adc-a0, predstavlja napon
Izlaz:  izračunati otpor senzora
Opis:   Naponsko djelilo od otpora senzora i tereta. Znamo napon pa time računamo rez.

************************************************************************************/ 
float MQResistanceCalculation(int raw_adc){
  
  return  ((float)RL_VALUE*(1023-raw_adc)/raw_adc);
}

/***************************** MQCalibration ****************************************
Ulaz:   mq_pin - analogni kanal
Izlaz:  Ro senzora
Opis:   Funkicja pretpostavlja da je senzor u čistome zraku.Koristi MQResistanceCalculation
        za izračun otpora senzora u čistom zraku te ga dijeli s RO_CLEAN_AIR_FACTOR.

************************************************************************************/ 
float MQCalibration(int mq_pin){
  
  int i;
  float val=0;

  for (i=0;i<CALIBARAION_SAMPLE_TIMES;i++) {           
    val += MQResistanceCalculation(analogRead(mq_pin));
    delay(CALIBRATION_SAMPLE_INTERVAL);
  }
  val = val/CALIBARAION_SAMPLE_TIMES;                 
  val = val/RO_CLEAN_AIR_FACTOR;                       
  return val; 
}
/*****************************  MQRead *********************************************
Ulaz:   mq_pin - analogni kanal (4)
Izlaz:  Rs - otpor senzora
Opis:   Koristi  MQResistanceCalculation za izračun otpora senzora (Rs).
        Rs se mijenja ovisno o promjeni koncentracije mjerenog plina.
************************************************************************************/ 
float MQRead(int mq_pin){
  
  int i;
  float rs=0;

  for (i=0;i<READ_SAMPLE_TIMES;i++) {
    rs += MQResistanceCalculation(analogRead(mq_pin));
    delay(READ_SAMPLE_INTERVAL);
  }
  rs = rs/READ_SAMPLE_TIMES;
  return rs;  
}

/*****************************  MQGetPercentage **********************************
Ulaz:   rs_ro_ratio - Rs/Ro
        pcurve      - pointer na krivulju ovisnosti dima
Uzlaz:  ppm traženog plina
Opis:   X (logaritamska vrijednost u ppm)može se izvesti ako je rs_ro_ratio zadan.
        Kako su vrijednosti koordinata u log mjeri, koristimo funkciju 10^X radi
        pretvorbe u ne log mjeru. 
************************************************************************************/ 
int  MQGetPercentage(float rs_ro_ratio, float *pcurve)
{
  return (pow(10,( ((log(rs_ro_ratio)-pcurve[1])/pcurve[2]) + pcurve[0])));
}
